; -- Festival.iss --

; INNO Setup Script File
; SEE THE DOCUMENTATION FOR DETAILS ON CREATING .ISS SCRIPT FILES!

; Inno 5:
; * {pf}
; * {pf32}
; * Deactivate the line "UsedUserAreasWarning=no".

; Inno 6:
; * {commonpf64}
; * {commonpf32}
; * Activate the line "UsedUserAreasWarning=no".

[Setup]
AppName=Festival - Sokoban Puzzle Solver
AppVerName=Festival - Sokoban Puzzle Solver
AppCopyright=Copyright  2020-2024 by Yaron Shoham
DefaultDirName={pf64}\Games\Sokoban\Festival
;DefaultDirName={localappdata}\Programs\Games\Sokoban\Festival
DefaultGroupName=Festival
;UninstallDisplayIcon={app}\Festival.exe
AppMutex=BDSokobanYASCMutex,Global\BDSokobanYASCMutex
DirExistsWarning=no
DisableStartupPrompt=yes
DisableDirPage=yes
DisableProgramGroupPage=yes
DisableFinishedPage=no
WizardStyle=modern
WindowVisible=no
;SetupIconFile=Setup.ico
PrivilegesRequired=admin
;UsedUserAreasWarning=no
UsePreviousAppDir=no
OutputBaseFilename=Festival_YASC_installation

[Files]
Source: "Festival.exe"; DestDir: "{app}"
Source: "libwinpthread-1.dll"; DestDir: "{app}"
Source: "README.txt"; DestDir: "{app}"; Flags: isreadme skipifsourcedoesntexist
Source: "License.txt"; DestDir: "{app}"; Flags: skipifsourcedoesntexist

; don't overwrite the user's settings file:
;Source: "Festival Settings.txt"; DestDir: "{userappdata}\Programs\Games\Sokoban\Festival"; Flags: onlyifdoesntexist skipifsourcedoesntexist

; don't overwrite the existing script file(s)
;Source: "Festival Proxy File.lsp"; DestDir: "{pf32}\BDSokobanYASC\Plugins"; DestName: "Festival.lsp"; Flags: onlyifdoesntexist skipifsourcedoesntexist
;Source: "Festival.lsp"; DestDir: "{userdocs}\Sokoban\Sokoban YASC\Lisp Scripts\Plugins"; Flags: onlyifdoesntexist skipifsourcedoesntexist
;Source: "Festival Settings.lsp"; DestDir: "{userdocs}\Sokoban\Sokoban YASC\Lisp Scripts\Plugins"; Flags: skipifsourcedoesntexist

;Source: "msvcr120.dll"; DestDir: "{sys}"; Flags: onlyifdoesntexist uninsneveruninstall

[Icons]

[Registry]

[Run]

[UninstallDelete] 
Type: files; Name: "{app}\*.*"
Type: dirifempty; Name: "{app}"

Type: filesandordirs; Name: "{userappdata}\Programs\Games\Sokoban\Festival\*.*"
Type: dirifempty; Name: "{userappdata}\Programs\Games\Sokoban\Festival"
Type: dirifempty; Name: "{userappdata}\Programs\Games\Sokoban"
Type: dirifempty; Name: "{userappdata}\Programs\Games"
Type: dirifempty; Name: "{userappdata}\Programs"

[Code]

function  AddSolverToYASCSolverList : Boolean;
var 
  Index : Integer;
  PluginFileName, IniFileName, IniSolverSection, StringValue : String;
begin
  Result := True;

  PluginFileName := ExpandConstant( '{pf32}\BDSokobanYASC\Plugins\Festival.lsp' );
  IniFileName := ExpandConstant( '{userdocs}\Sokoban\Sokoban YASC\Sokoban.ini' );
  IniSolverSection := 'Solver';

  //MsgBox( IniFileName, mbInformation, MB_OK );
  //MsgBox( PluginFileName, mbInformation, MB_OK );

  if FileExists( IniFileName ) then begin // 'True': Sokoban YASC is installed on the computer
     Index := 0;
     repeat Index := Index + 1;
            StringValue := GetIniString( IniSolverSection, IntToStr( Index ), '', IniFileName );
     until  ( StringValue = '' ) or ( StringValue = PluginFileName );

     if StringValue = '' then // 'True': Festival is not on the list of installed solvers for Sokoban YASC
        if   SetIniString( IniSolverSection, IntToStr( Index ), PluginFileName, IniFileName ) then begin
             if MsgBox( 'Do you want to make Festival the currently selected solver for Sokoban YASC?' #13#13 'You do not have to do it now. You can always select which solver to use from the list of installed solvers.', mbConfirmation, MB_YESNO +  MB_DEFBUTTON2 ) = idYes then
                if   SetIniString( IniSolverSection, 'FileName', PluginFileName, IniFileName ) then {OK}
                else MsgBox( 'Registering Festival in YASC as the currently selected solver failed', mbError, MB_OK );
             end
        else MsgBox( 'Registering Festival in Sokoban YASC failed', mbError, MB_OK );
     end;
end;

function  ExpandExeFileNameInSettingsFile : Boolean;
var
  FileName, Text, ProgramFolder, ProgramFolder32Bit : String;
  AnsiText : AnsiString;
begin
  Result := False;
  ProgramFolder := ExpandConstant ( '{pf64}\Games\Sokoban\Festival' ); // ExpandConstant( '{app}' );
  StringChangeEx( ProgramFolder, '\', '\\', True );
  ProgramFolder32Bit := ExpandConstant( '{pf32}\Festival Sokoban Solver' ); // Festival 1.0 was a 32-bit application
  StringChangeEx( ProgramFolder32Bit, '\', '\\', True );

  FileName := ExpandConstant( '{userappdata}\Programs\Games\Sokoban\Festival\Festival Settings.txt' );
  if FileExists( FileName ) and
     LoadStringFromFile( FileName, AnsiText ) then begin
     Text := AnsiText;
     StringChangeEx( Text, '{ProgramFolder}', ProgramFolder, True );
     StringChangeEx( Text, ProgramFolder32Bit, ProgramFolder, True );
     AnsiText := Text;
     Result := SaveStringToFile( FileName, AnsiText, False );
     end;
end;

function  Delete32BitVersion : Boolean; // Festival 1.0 was a 32-bit application
var
  ProgramFolder32Bit : String;
begin
  ProgramFolder32Bit := ExpandConstant( '{pf32}\Festival Sokoban Solver' );
  Result := not DirExists( ProgramFolder32Bit );

  if not Result then begin
     DeleteFile( ProgramFolder32Bit + '\\Festival.exe' );
     DeleteFile( ProgramFolder32Bit + '\\unins000.dat' );
     DeleteFile( ProgramFolder32Bit + '\\unins000.exe' );
     Result := RemoveDir( ProgramFolder32Bit );
     end;
end;

procedure CurStepChanged(CurStep: TSetupStep);
begin
  if CurStep = ssPostInstall then begin
     ExpandExeFileNameInSettingsFile;
     AddSolverToYASCSolverList;
     Delete32BitVersion;
     end;
end;
